package com.sitemesh.filter;

import com.sitemesh.*;
import java.io.IOException;
import javax.servlet.*;
import javax.servlet.http.*;

/**
 * Main SiteMesh filter for applying Decorators to entire Pages.
 *
 * @author <a href="joe@truemesh.com">Joe Walnes</a>
 * @version $Revision: 1.11 $
 */
public class PageFilter implements Filter, RequestConstants {

	private FilterConfig filterConfig;
	private DecoratorMapper mapper;

	/**
	 * @label wraps response with
	 * @directed
	 */
	/*#private PageResponse lnkPageResponse;*/

	/**
	 * Main method of Filter.
	 *
	 * <p>Checks if Filter has been applied this request. If not, parses the Page
	 * and applies Decorator (if found).
	 */
	public void doFilter( ServletRequest rq,
	                      ServletResponse rs,
	                      FilterChain chain )
	                      throws IOException, ServletException {

		HttpServletRequest request = (HttpServletRequest) rq;
		HttpServletResponse response = (HttpServletResponse) rs;

		if ( request.getAttribute( FILTER_APPLIED ) != null ) {
			// Ensure that filter is only applied once per request.
			chain.doFilter( request, response );
		}
		else {

			request.setAttribute( FILTER_APPLIED, Boolean.TRUE );

			// Parse data into Page object
			Page page = parsePage( request, response, chain );
			page.setRequest( request );

			// Get Decorator
			Decorator decorator = mapper.getDecorator( request, page );

			// Generate result
			if ( decorator != null ) {
				applyDecorator( page, decorator, request, response );
			}
			else {
				writeOriginal(response, page);
			}

		}
	}

	/**
	 * Standard Filter method.
	 */
	public FilterConfig getFilterConfig() {
		return filterConfig;
	}

	/**
	 * Set FilterConfig, and get instance of {@link com.sitemesh.DecoratorMapper} .
	 */
	public void setFilterConfig( FilterConfig filterConfig ) {
		this.filterConfig = filterConfig;
		mapper = Factory.getInstance(filterConfig).getDecoratorMapper();
	}

	/**
	 * Continue in filter-chain, writing all content to buffer and parsing
	 * into returned Page object.
	 */
	private Page parsePage(HttpServletRequest request, HttpServletResponse response, FilterChain chain) throws IOException, ServletException {
		PageResponse pageResponse = new PageResponse( response, filterConfig );
		chain.doFilter( request, pageResponse );
		Page result = pageResponse.getPage();
		pageResponse.closeStream();
		return result;
	}

	/**
	 * Write the original page data to the response.
	 */
	private void writeOriginal( HttpServletResponse response, Page page) throws IOException {
		response.setContentLength( page.getContentLength() );
		page.writePage( response.getOutputStream() );
	}

	/**
	 * Apply Decorator to Page and write to response
	 */
	private void applyDecorator( Page page, Decorator decorator, HttpServletRequest request, HttpServletResponse response ) throws ServletException, IOException {
		request.setAttribute( PAGE, page );
		ServletContext context = getFilterConfig().getServletContext();
		RequestDispatcher dispatcher = context.getRequestDispatcher( decorator.getPage() );
		dispatcher.forward( request, response );
	}

}
